from os.path import join

from guet import constants
from guet.config import CONFIGURATION_DIRECTORY


def most_recent_committers_set():
    set_committers_file = open(join(CONFIGURATION_DIRECTORY, constants.COMMITTERS_SET), 'r')
    line = set_committers_file.readline()
    set_committers_file.close()
    split = line.rstrip().split(',')
    return int(split[len(split)-1])



from typing import List

from guet.commands.command import Command
from guet.commands.help_message_strategy import HelpMessageBuilder
from guet.settings.settings import Settings
from guet.commands.command_factory import CommandFactoryMethod
from guet.commands.strategy_command import StrategyCommand
from guet.commands.too_few_args import TooFewArgsStrategy
from guet.commands.strategy import CommandStrategy
from guet.commands.too_many_args import TooManyArgsStrategy
from guet.commands.addcommitter.add_committer_strategy import AddCommitterStrategy

ADD_COMMITTER_HELP_MESSAGE = HelpMessageBuilder('guet add <initials> <"name"> <email>',
                                                "Add committer to make available for commit tracking.").build()


class AddCommitterFactory(CommandFactoryMethod):

    def build(self, args: List[str], settings: Settings) -> Command:
        strategy = self._choose_strategy(args[1:], settings)
        return StrategyCommand(strategy)

    def short_help_message(self):
        return 'Add committer to the list of available committers'

    def _choose_strategy(self, args: List[str], _: Settings) -> CommandStrategy:
        if len(args) < 3:
            return TooFewArgsStrategy(ADD_COMMITTER_HELP_MESSAGE)
        elif len(args) > 3:
            return TooManyArgsStrategy()
        else:
            return AddCommitterStrategy(args[0], args[1], args[2])



from typing import List

from guet.commands.argsettingcommand import ArgSettingCommand
from guet.commands.command import Command
from guet.commands.command_factory import CommandFactoryMethod
from guet.commands.help.guet_usage import guet_usage
from guet.commands.help_message_strategy import HelpMessageStrategy
from guet.commands.strategy_command import StrategyCommand
from guet.settings.settings import Settings
from guet.config.get_config import get_config
from guet.config.already_initialized import already_initialized


class CommandFactory:
    def __init__(self, command_builder_map):
        self.command_builder_map = command_builder_map

    def create(self, args: list) -> ArgSettingCommand:
        result = None
        if already_initialized():
            result = self._create_with_settings(args, get_config())
        else:
            result = self._create_with_settings(args, Settings())
        return result

    def _create_with_settings(self, args: list, settings: Settings) -> Command:
        if len(args) > 0:
            command_arg = args[0]
            command_type = self.command_builder_map[command_arg]
            if isinstance(command_type, CommandFactoryMethod):
                return self._create_with_command_factory(command_type, args, settings)
            else:
                return self._create_with_command_constructor(command_type, args, settings)

        else:
            return StrategyCommand(HelpMessageStrategy(guet_usage(self.command_builder_map)))

    def _create_with_command_constructor(self, command_class, args: List[str],
                                         settings: Settings) -> ArgSettingCommand:
        return command_class(args, settings)

    def _create_with_command_factory(self, command_factory: CommandFactoryMethod, args: List[str],
                                     settings: Settings) -> ArgSettingCommand:
        return command_factory.build(args, settings)



from typing import List

from guet.commands.strategy import CommandStrategy
from guet.settings.settings import Settings


class PrintCommandStrategy(CommandStrategy):
    def __init__(self, text: str):
        self._text = text

    def apply(self):
        print(self._text)



from typing import List

from guet.commands.strategy import CommandStrategy
from guet.git._hook_mode import HookMode
from guet.git.any_hooks_present import any_hooks_present
from guet.git.create_hook import create_hook, Hooks
from guet.git.git_path_from_cwd import git_hook_path_from_cwd
from guet.git.git_present_in_cwd import git_present_in_cwd
from guet.settings.settings import Settings


class StartCommandStrategy(CommandStrategy):

    def apply(self):
        if git_present_in_cwd():
            hook_path = git_hook_path_from_cwd()
            if not any_hooks_present(hook_path):
                self._create_all_hooks(hook_path, HookMode.NEW_OR_OVERWRITE)
            else:
                hook_mode = None
                print(('There is already commit hooks in this project. ' +
                       'Would you like to overwrite (o), ' +
                       'create (c) the file and put it in the hooks folder, or cancel (x)?'))
                val = input()
                if val == 'o':
                    hook_mode = HookMode.NEW_OR_OVERWRITE
                elif val == 'c':
                    hook_mode = HookMode.CREATE_ALONGSIDE
                else:
                    hook_mode = HookMode.CANCEL
                self._create_all_hooks(hook_path, hook_mode)

        else:
            print('Git not initialized in this directory.')

    @staticmethod
    def _create_all_hooks(hook_folder_path: str, hook_mode: HookMode):
        if hook_mode is not HookMode.CANCEL:
            create_hook(hook_folder_path, Hooks.PRE_COMMIT, hook_mode)
            create_hook(hook_folder_path, Hooks.POST_COMMIT, hook_mode)
            create_hook(hook_folder_path, Hooks.COMMIT_MSG, hook_mode)



from guet.config.get_config import get_config
from guet.config.most_recent_committers_set import most_recent_committers_set
from guet.currentmillis import current_millis
from guet.util.errors import log_on_error


@log_on_error
def pre_commit():    
    _check_within_timeframe_if_enabled()

def _check_within_timeframe_if_enabled():
    settings = get_config()
    if settings.read('pairReset'):
        _fail_if_past_valid_timeframe()

def _fail_if_past_valid_timeframe():
    now = current_millis()
    twenty_four_hours = 86400000
    twenty_four_hours_ago = now - twenty_four_hours
    set_time = most_recent_committers_set()
    if set_time < twenty_four_hours_ago:
        print(("\nYou have not reset pairs in over twenty four hours!\n" +
               "Please reset your pairs by using guet set and including your pairs' initials\n"))
        exit(1)



from typing import List

from guet.commands.command import Command
from guet.settings.settings import Settings


class ArgSettingCommand(Command):
    @classmethod
    def get_short_help_message(cls):
        return cls.help_short()

    def __init__(self, args: List[str], settings: Settings, args_needed: bool = True):
        self.args_needed = args_needed
        self.args = args[1:]
        self.settings = settings

    def execute(self) -> None:
        no_args_given = len(self.args) == 0
        if no_args_given and self.args_needed:
            self._print_help_message()
        else:
            self.execute_hook()

    def execute_hook(self) -> None:
        raise NotImplementedError

    def help(self) -> str:
        raise NotImplementedError

    @classmethod
    def help_short(cls) -> str:
        raise NotImplementedError

    def _print_help_message(self) -> None:
        print(f'{self.help()}\n')



from typing import List

from guet.commands.strategy import CommandStrategy
from guet.settings.settings import Settings


class HelpMessageBuilder:

    def __init__(self, usage: str, description: str):
        self._usage = usage
        self._description = description
        self._explanation = None

    def explanation(self, explanation: str):
        self._explanation = explanation
        return self

    def build(self) -> str:
        message = f'usage: {self._usage}\n\n{self._description}'
        if self._explanation is not None:
            message += f'\n\n{self._explanation}'
        return f'{message}\n'


class HelpMessageStrategy(CommandStrategy):
    def __init__(self, help_message: str):
        self.help_message = help_message

    def apply(self):
        print(self.help_message)



from typing import List

from guet.commands.argsettingcommand import ArgSettingCommand
from guet.commands.command import Command
from guet.commands.command_factory import CommandFactoryMethod
from guet.commands.help_message_strategy import HelpMessageBuilder
from guet.commands.start.start_strategy import StartCommandStrategy
from guet.commands.strategy_command import StrategyCommand
from guet.settings.settings import Settings

START_HELP_MESSAGE = HelpMessageBuilder('guet start', 'Initialize current .git project to use guet.').build()


class StartCommandFactory(CommandFactoryMethod):
    def short_help_message(self) -> str:
        return 'Start guet usage in the repository at current directory'

    def build(self, args: List[str], settings: Settings) -> Command:
        return StrategyCommand(StartCommandStrategy())



from setuptools import setup

import guet

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(name='guet',
      version=guet.__version__,
      description='Enable contribution tracking when pair programming',
      long_description=long_description,
      long_description_content_type="text/markdown",
      url='https://github.com/chiptopher/guet',
      keywords='pair programming',
      entry_points={'console_scripts': ['guet=guet.main:main']},
      packages=[
          'guet',
          'guet.commands', 'guet.commands.get', 'guet.commands.addcommitter', 'guet.commands.init',
          'guet.commands.start',
          'guet.commands.config', 'guet.hooks', 'guet.commands.config', 'guet.commands.config', 'guet.commands.help',
          'guet.commands.setcommitters', 'guet.files', 'guet.settings', 'guet.git', 'guet.config',
          'guet.util'
      ])



import unittest
from typing import List
from unittest.mock import patch
from guet.commands.help.guet_usage import guet_usage
from guet.commands.argsettingcommand import ArgSettingCommand
from guet.commands.command_factory import CommandFactoryMethod
from guet.factory import CommandFactory
from guet.settings.settings import Settings


@patch('guet.factory.already_initialized', return_value=True)
@patch('guet.factory.get_config')
class TestCommandFactory(unittest.TestCase):
    def test_returns_command_from_builder_method_that_matches(self, mock_get_settings,
                                                              mock_already_init):
        builder_map = dict()
        builder_map['command'] = MockCommand
        builder_map['not-command'] = NotMockCommand

        command_factory = CommandFactory(builder_map)
        args = ['command']
        result = command_factory.create(args)
        self.assertEqual(MockCommand, type(result))

    def test_returns_command_with_settings_from_settings_file(self, mock_get_settings,
                                                              mock_already_init):
        expected_settings = Settings()
        mock_get_settings.return_value = expected_settings
        builder_map = dict()
        builder_map['command'] = MockCommand
        builder_map['not-command'] = NotMockCommand

        command_factory = CommandFactory(builder_map)
        args = ['command']
        result = command_factory.create(args)
        self.assertEqual(expected_settings, result.settings)

    def test_uses_new_settings_with_default_values_if_initialization_hasnt_been_ran(
            self, mock_get_settings, mock_already_init):
        mock_already_init.return_value = False
        returned_settings = Settings()
        mock_get_settings.return_value = returned_settings
        builder_map = dict()
        builder_map['command'] = MockCommand

        command_factory = CommandFactory(builder_map)
        args = ['command']
        result = command_factory.create(args)
        self.assertNotEqual(returned_settings, result.settings)

    @patch('builtins.print')
    def test_uses_command_builder_map_to_print_help_messages(self, mock_print, mock_get_settings,
                                                             mock_already_init):
        builder_map = dict()
        builder_map['command'] = MockCommandFactory()
        command_factory = CommandFactory(builder_map)
        result = command_factory.create([])
        result.execute()
        mock_print.assert_called_with(guet_usage(builder_map))

    def test_returns_command_using_command_factory_build_method(self, mock_get_settings,
                                                                mock_already_init):
        builder_map = dict()
        builder_map['command'] = MockCommandFactory()
        builder_map['not-command'] = NotMockCommand

        command_factory = CommandFactory(builder_map)
        args = ['command']
        result = command_factory.create(args)
        self.assertEqual(MockCommand, type(result))


class MockCommand(ArgSettingCommand):
    def execute_hook(self) -> None:
        pass

    def help(self) -> str:
        pass

    @classmethod
    def help_short(cls) -> str:
        pass


class NotMockCommand(ArgSettingCommand):
    def execute_hook(self) -> None:
        pass

    def help(self) -> str:
        pass

    @classmethod
    def help_short(cls) -> str:
        pass


class MockCommandFactory(CommandFactoryMethod):
    def short_help_message(self):
        pass

    def build(self, args: List[str], settings: Settings):
        return MockCommand(args, settings)



from typing import List

from guet.settings.settings import Settings
from guet.commands.argsettingcommand import ArgSettingCommand


class CommandFactoryMethod:
    def short_help_message(self) -> str:
        raise NotImplementedError

    def build(self, args: List[str], settings: Settings) -> ArgSettingCommand:
        raise NotImplementedError



import sys

from guet.commands.get.get_factory import GetCommandFactory, GET_HELP_MESSAGE
from guet.commands.help_decorator import HelpDecorator
from guet.commands.init_required_decorator import InitRequiredDecorator
from guet.commands.start.factory import StartCommandFactory, START_HELP_MESSAGE
from guet.factory import CommandFactory
from guet.util.errors import log_on_error
from guet.commands.addcommitter.factory import AddCommitterFactory, ADD_COMMITTER_HELP_MESSAGE
from guet.commands.init.factory import InitCommandFactory, INIT_HELP_MESSAGE
from guet.commands.setcommitters.factory import SetCommittersCommandFactory, SET_HELP_MESSAGE
from guet.commands.config.factory import ConfigCommandFactory, CONFIG_HELP_MESSAGE


def _command_builder_map():
    command_builder_map = dict()
    command_builder_map['add'] = InitRequiredDecorator(HelpDecorator(AddCommitterFactory(), ADD_COMMITTER_HELP_MESSAGE))
    command_builder_map['init'] = HelpDecorator(InitCommandFactory(), INIT_HELP_MESSAGE, no_args_valid=True)
    command_builder_map['set'] = InitRequiredDecorator(HelpDecorator(SetCommittersCommandFactory(), SET_HELP_MESSAGE))
    command_builder_map['start'] = InitRequiredDecorator(
        HelpDecorator(StartCommandFactory(), START_HELP_MESSAGE, no_args_valid=True))
    command_builder_map['config'] = InitRequiredDecorator(HelpDecorator(ConfigCommandFactory(), CONFIG_HELP_MESSAGE))
    command_builder_map['get'] = InitRequiredDecorator(HelpDecorator(GetCommandFactory(), GET_HELP_MESSAGE))
    return command_builder_map


@log_on_error
def main():
    command_factory = CommandFactory(_command_builder_map())
    command = command_factory.create(sys.argv[1:])
    command.execute()
